#!/usr/bin/env python
# -*- coding: utf-8 -*-

from settings import *

import argparse
from datetime import datetime
import os, shutil
import shlex, subprocess
import pysam

from settings import *

parser = argparse.ArgumentParser(description='Annotate a VCF File with DbNSFP.')

parser.add_argument('-i', dest='vcffile', required=True, metavar='example.vcf', help='a VCF file to be annotated')

args = parser.parse_args()

dbnfsp_header = '''##INFO=<ID=dbNSFP_GERP++_RS,Number=A,Type=Float,Description="Field 'GERP++_RS' from dbNSFP">
##INFO=<ID=dbNSFP_1000Gp1_ASN_AF,Number=A,Type=Float,Description="Field '1000Gp1_ASN_AF' from dbNSFP">
##INFO=<ID=dbNSFP_Ensembl_geneid,Number=A,Type=String,Description="Field 'Ensembl_geneid' from dbNSFP">
##INFO=<ID=dbNSFP_CADD_phred,Number=A,Type=Float,Description="Field 'CADD_phred' from dbNSFP">
##INFO=<ID=dbNSFP_FATHMM_score,Number=A,Type=Float,Description="Field 'FATHMM_score' from dbNSFP">
##INFO=<ID=dbNSFP_1000Gp1_ASN_AC,Number=A,Type=Integer,Description="Field '1000Gp1_ASN_AC' from dbNSFP">
##INFO=<ID=dbNSFP_GERP++_RS_rankscore,Number=A,Type=Float,Description="Field 'GERP++_RS_rankscore' from dbNSFP">
##INFO=<ID=dbNSFP_MutationTaster_pred,Number=A,Type=Character,Description="Field 'MutationTaster_pred' from dbNSFP">
##INFO=<ID=dbNSFP_SiPhy_29way_logOdds_rankscore,Number=A,Type=Float,Description="Field 'SiPhy_29way_logOdds_rankscore' from dbNSFP">
##INFO=<ID=dbNSFP_phastCons46way_placental_rankscore,Number=A,Type=Float,Description="Field 'phastCons46way_placental_rankscore' from dbNSFP">
##INFO=<ID=dbNSFP_FATHMM_pred,Number=A,Type=Character,Description="Field 'FATHMM_pred' from dbNSFP">
##INFO=<ID=dbNSFP_phastCons100way_vertebrate,Number=A,Type=Float,Description="Field 'phastCons100way_vertebrate' from dbNSFP">
##INFO=<ID=dbNSFP_MutationTaster_score,Number=A,Type=Float,Description="Field 'MutationTaster_score' from dbNSFP">
##INFO=<ID=dbNSFP_VEST3_rankscore,Number=A,Type=Float,Description="Field 'VEST3_rankscore' from dbNSFP">
##INFO=<ID=dbNSFP_fold-degenerate,Number=A,Type=Integer,Description="Field 'fold-degenerate' from dbNSFP">
##INFO=<ID=dbNSFP_1000Gp1_AMR_AC,Number=A,Type=Integer,Description="Field '1000Gp1_AMR_AC' from dbNSFP">
##INFO=<ID=dbNSFP_MutationAssessor_score,Number=A,Type=Float,Description="Field 'MutationAssessor_score' from dbNSFP">
##INFO=<ID=dbNSFP_ESP6500_EA_AF,Number=A,Type=Float,Description="Field 'ESP6500_EA_AF' from dbNSFP">
##INFO=<ID=dbNSFP_phyloP46way_primate_rankscore,Number=A,Type=Float,Description="Field 'phyloP46way_primate_rankscore' from dbNSFP">
##INFO=<ID=dbNSFP_LRT_pred,Number=A,Type=Character,Description="Field 'LRT_pred' from dbNSFP">
##INFO=<ID=dbNSFP_phastCons100way_vertebrate_rankscore,Number=A,Type=Float,Description="Field 'phastCons100way_vertebrate_rankscore' from dbNSFP">
##INFO=<ID=dbNSFP_1000Gp1_AMR_AF,Number=A,Type=Float,Description="Field '1000Gp1_AMR_AF' from dbNSFP">
##INFO=<ID=dbNSFP_LRT_Omega,Number=A,Type=Float,Description="Field 'LRT_Omega' from dbNSFP">
##INFO=<ID=dbNSFP_GERP++_NR,Number=A,Type=Float,Description="Field 'GERP++_NR' from dbNSFP">
##INFO=<ID=dbNSFP_FATHMM_rankscore,Number=A,Type=Float,Description="Field 'FATHMM_rankscore' from dbNSFP">
##INFO=<ID=dbNSFP_Interpro_domain,Number=A,Type=String,Description="Field 'Interpro_domain' from dbNSFP">
##INFO=<ID=dbNSFP_MutationAssessor_pred,Number=A,Type=Character,Description="Field 'MutationAssessor_pred' from dbNSFP">
##INFO=<ID=dbNSFP_SiPhy_29way_logOdds,Number=A,Type=Float,Description="Field 'SiPhy_29way_logOdds' from dbNSFP">
##INFO=<ID=dbNSFP_LRT_score,Number=A,Type=Float,Description="Field 'LRT_score' from dbNSFP">
##INFO=<ID=dbNSFP_phyloP100way_vertebrate,Number=A,Type=Float,Description="Field 'phyloP100way_vertebrate' from dbNSFP">
##INFO=<ID=dbNSFP_cds_strand,Number=A,Type=Character,Description="Field 'cds_strand' from dbNSFP">
##INFO=<ID=dbNSFP_VEST3_score,Number=A,Type=Float,Description="Field 'VEST3_score' from dbNSFP">
##INFO=<ID=dbNSFP_RadialSVM_score,Number=A,Type=Float,Description="Field 'RadialSVM_score' from dbNSFP">
##INFO=<ID=dbNSFP_RadialSVM_pred,Number=A,Type=Character,Description="Field 'RadialSVM_pred' from dbNSFP">
##INFO=<ID=dbNSFP_LR_score,Number=A,Type=Float,Description="Field 'LR_score' from dbNSFP">
##INFO=<ID=dbNSFP_LR_pred,Number=A,Type=Character,Description="Field 'LR_pred' from dbNSFP">
##INFO=<ID=dbNSFP_phastCons46way_placental,Number=A,Type=Float,Description="Field 'phastCons46way_placental' from dbNSFP">
##INFO=<ID=dbNSFP_CADD_raw_rankscore,Number=A,Type=Float,Description="Field 'CADD_raw_rankscore' from dbNSFP">
##INFO=<ID=dbNSFP_SIFT_converted_rankscore,Number=A,Type=Float,Description="Field 'SIFT_converted_rankscore' from dbNSFP">
##INFO=<ID=dbNSFP_phyloP100way_vertebrate_rankscore,Number=A,Type=Float,Description="Field 'phyloP100way_vertebrate_rankscore' from dbNSFP">
##INFO=<ID=dbNSFP_MutationAssessor_rankscore,Number=A,Type=Float,Description="Field 'MutationAssessor_rankscore' from dbNSFP">
##INFO=<ID=dbNSFP_Uniprot_acc,Number=A,Type=String,Description="Field 'Uniprot_acc' from dbNSFP">
##INFO=<ID=dbNSFP_codonpos,Number=A,Type=Integer,Description="Field 'codonpos' from dbNSFP">
##INFO=<ID=dbNSFP_Polyphen2_HDIV_score,Number=A,Type=Float,Description="Field 'Polyphen2_HDIV_score' from dbNSFP">
##INFO=<ID=dbNSFP_UniSNP_ids,Number=A,Type=String,Description="Field 'UniSNP_ids' from dbNSFP">
##INFO=<ID=dbNSFP_SIFT_pred,Number=A,Type=Character,Description="Field 'SIFT_pred' from dbNSFP">
##INFO=<ID=dbNSFP_SiPhy_29way_pi,Number=A,Type=String,Description="Field 'SiPhy_29way_pi' from dbNSFP">
##INFO=<ID=dbNSFP_LR_rankscore,Number=A,Type=Float,Description="Field 'LR_rankscore' from dbNSFP">
##INFO=<ID=dbNSFP_CADD_raw,Number=A,Type=Float,Description="Field 'CADD_raw' from dbNSFP">
##INFO=<ID=dbNSFP_LRT_converted_rankscore,Number=A,Type=Float,Description="Field 'LRT_converted_rankscore' from dbNSFP">
##INFO=<ID=dbNSFP_aapos_FATHMM,Number=A,Type=String,Description="Field 'aapos_FATHMM' from dbNSFP">
##INFO=<ID=dbNSFP_aapos,Number=A,Type=Integer,Description="Field 'aapos' from dbNSFP">
##INFO=<ID=dbNSFP_1000Gp1_AFR_AF,Number=A,Type=Float,Description="Field '1000Gp1_AFR_AF' from dbNSFP">
##INFO=<ID=dbNSFP_1000Gp1_AFR_AC,Number=A,Type=Integer,Description="Field '1000Gp1_AFR_AC' from dbNSFP">
##INFO=<ID=dbNSFP_refcodon,Number=A,Type=String,Description="Field 'refcodon' from dbNSFP">
##INFO=<ID=dbNSFP_SLR_test_statistic,Number=A,Type=Float,Description="Field 'SLR_test_statistic' from dbNSFP">
##INFO=<ID=dbNSFP_Uniprot_aapos,Number=A,Type=Integer,Description="Field 'Uniprot_aapos' from dbNSFP">
##INFO=<ID=dbNSFP_1000Gp1_AC,Number=A,Type=Integer,Description="Field '1000Gp1_AC' from dbNSFP">
##INFO=<ID=dbNSFP_1000Gp1_AF,Number=A,Type=Float,Description="Field '1000Gp1_AF' from dbNSFP">
##INFO=<ID=dbNSFP_aapos_SIFT,Number=A,Type=String,Description="Field 'aapos_SIFT' from dbNSFP">
##INFO=<ID=dbNSFP_SIFT_score,Number=A,Type=Float,Description="Field 'SIFT_score' from dbNSFP">
##INFO=<ID=dbNSFP_Reliability_index,Number=A,Type=Integer,Description="Field 'Reliability_index' from dbNSFP">
##INFO=<ID=dbNSFP_MutationTaster_converted_rankscore,Number=A,Type=Float,Description="Field 'MutationTaster_converted_rankscore' from dbNSFP">
##INFO=<ID=dbNSFP_Uniprot_id,Number=A,Type=String,Description="Field 'Uniprot_id' from dbNSFP">
##INFO=<ID=dbNSFP_Ensembl_transcriptid,Number=A,Type=String,Description="Field 'Ensembl_transcriptid' from dbNSFP">
##INFO=<ID=dbNSFP_phastCons46way_primate,Number=A,Type=Float,Description="Field 'phastCons46way_primate' from dbNSFP">
##INFO=<ID=dbNSFP_ESP6500_AA_AF,Number=A,Type=Float,Description="Field 'ESP6500_AA_AF' from dbNSFP">
##INFO=<ID=dbNSFP_RadialSVM_rankscore,Number=A,Type=Float,Description="Field 'RadialSVM_rankscore' from dbNSFP">
##INFO=<ID=dbNSFP_Polyphen2_HVAR_score,Number=A,Type=Float,Description="Field 'Polyphen2_HVAR_score' from dbNSFP">
##INFO=<ID=dbNSFP_Ancestral_allele,Number=A,Type=Character,Description="Field 'Ancestral_allele' from dbNSFP">
##INFO=<ID=dbNSFP_phyloP46way_placental,Number=A,Type=Float,Description="Field 'phyloP46way_placental' from dbNSFP">
##INFO=<ID=dbNSFP_phyloP46way_primate,Number=A,Type=Float,Description="Field 'phyloP46way_primate' from dbNSFP">
##INFO=<ID=dbNSFP_Polyphen2_HDIV_pred,Number=A,Type=Character,Description="Field 'Polyphen2_HDIV_pred' from dbNSFP">
##INFO=<ID=dbNSFP_Polyphen2_HDIV_rankscore,Number=A,Type=Float,Description="Field 'Polyphen2_HDIV_rankscore' from dbNSFP">
##INFO=<ID=dbNSFP_1000Gp1_EUR_AF,Number=A,Type=Float,Description="Field '1000Gp1_EUR_AF' from dbNSFP">
##INFO=<ID=dbNSFP_phyloP46way_placental_rankscore,Number=A,Type=Float,Description="Field 'phyloP46way_placental_rankscore' from dbNSFP">
##INFO=<ID=dbNSFP_Polyphen2_HVAR_rankscore,Number=A,Type=Float,Description="Field 'Polyphen2_HVAR_rankscore' from dbNSFP">
##INFO=<ID=dbNSFP_phastCons46way_primate_rankscore,Number=A,Type=Float,Description="Field 'phastCons46way_primate_rankscore' from dbNSFP">
##INFO=<ID=dbNSFP_Polyphen2_HVAR_pred,Number=A,Type=Character,Description="Field 'Polyphen2_HVAR_pred' from dbNSFP">
##INFO=<ID=dbNSFP_1000Gp1_EUR_AC,Number=A,Type=Integer,Description="Field '1000Gp1_EUR_AC' from dbNSFP">
'''

class Dbnsfp(object):
    def __init__(self, vcffile=None):
        
        self.vcffile = vcffile

        self.filename = os.path.splitext(os.path.basename(str(vcffile)))[0]
        self.dbnfsp_reader = pysam.Tabixfile(dbnsfp)
        self.header = self.dbnfsp_reader.header.next().strip().split('\t')
        # for item in self.dbnfsp_reader.header:
        #     print 'item',item
        #     #self.header.append(item)
        #print self.header
        
        #create folder snpeff if it doesn't exists
        if not os.path.exists('dbnfsp'):
            os.makedirs('dbnfsp')
        #enter inside folder
        os.chdir('dbnfsp')
        
    
    def run(self):
        tstart = datetime.now()
        print(tstart, 'Starting dbnfsp annotation: ', self.vcffile)
        
        self.annotate()

        tend = datetime.now()
        annotation_time =  tend - tstart
        print(tend, 'Finished dbnfsp annotation, it took: ', annotation_time)

    def check_ref_alt(self, variant, ann):
        #compare REF
        if variant[3] == ann[2]:
            alts = variant[4].split(',')
            alts_ann = ann[3].split(',')
            #compare ALT
            for alt in alts:
                if alt in alts_ann:
                    return True
        return False
    #convert and annotate the vcf file to snpeff
    def annotate(self):
        #print 'Hello'
        #print self.dbnfsp_reader
        vcf_reader = open('../%s' % (self.vcffile))
        vcf_writer = open('dbnsfp.vcf', 'w')
        
        for line in vcf_reader:
            if line.startswith('#'):
                if line.startswith('#CHROM'):
                    vcf_writer.writelines(dbnfsp_header)
                vcf_writer.writelines(line)
            else:
                variant = line.split('\t')
                variant[0] = variant[0].replace('chr', '')
                index = '%s-%s' % (variant[0], variant[1])
                #print index
                try:
                    records = self.dbnfsp_reader.fetch(variant[0], int(variant[1])-1, int(variant[1]))
                except ValueError:
                    records = []
                for record in records:
                    ann = record.strip().split('\t')

                    if self.check_ref_alt(variant, ann):
                        new_ann = []
                        #this is the column from where to start annotation
                        start_index = 11
                        start_index = 108
                        #only freequencies 108 - 175
                        for k, item in enumerate(self.header[start_index:]):
                            # print k, item, ann[k+8]
                            if ann[k+start_index] != '.':
                                new_ann.append('dbnsfp.%s=%s' % (item, ann[k+start_index].replace(';', '|')))
                        variant[7] = '%s;%s' % (variant[7], ";".join(new_ann))
                vcf_writer.writelines("\t".join(variant))
                    


if  __name__ == '__main__' :
    dbnsfp = Dbnsfp(args.vcffile)
    dbnsfp.run()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
from datetime import datetime
import os, shutil
import shlex, subprocess
from subprocess import call
from pynnotator import settings

toolname = 'validator'

env = os.environ.copy()
env['PERL5LIB'] = settings.vcftools_dir_perl

class Validator(object):
    def __init__(self, vcf_file=None):
        
        self.vcf_file = vcf_file

        self.filename = os.path.splitext(os.path.basename(str(vcf_file)))[0]
        #create folder validator if it doesn't exists
        if not os.path.exists('validator'):
            os.makedirs('validator')
    
    def run(self):

        tstart = datetime.now()
        print(tstart, 'Starting validator: ', self.vcf_file)
        
        std = self.validate()

        tend = datetime.now()
        annotation_time =  tend - tstart
        print(tend, 'Finished validator, it took: ', annotation_time)        
        
        return std

    #Validate vcf file with Vcftools
    def validate(self):
        #check if file is in .gz format
        if not self.vcf_file.endswith('.gz'):
            command = '%s/bgzip -c %s > validator/%s.vcf.gz' % (settings.htslib_dir, self.vcf_file, self.filename)
            call(command, shell=True)
        else:
            command = 'cp %s validator/' % (self.vcf_file)
            call(command, shell=True)

        command = '%s/tabix -p vcf validator/%s.vcf.gz' % (settings.htslib_dir, self.filename)
        call(command, shell=True)

        command = '%s/vcf-validator validator/%s.vcf.gz 2>validator/validation.log' % (settings.vcftools_dir_perl, self.filename)

        # print 'validator command', command
        p = subprocess.call(command, 
            cwd=os.getcwd(), 
            env=env, 
            shell=True)

        time_end = datetime.now()
        
        if p == 0:
            
            print(time_end, 'This vcf was sucessfully validated by vcf-validator!')
        else:
            print(time_end, 'Sorry this vcf could not be validated!')
        return p

if  __name__ == '__main__' :
    parser = argparse.ArgumentParser(description='Annotate a VCF File with VAlIDATOR.')
    parser.add_argument('-i', dest='vcf_file', required=True, metavar='example.vcf', help='a VCF file to be annotated')

    args = parser.parse_args()
    validator = Validator(args.vcf_file)
    validator.run()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#parallel https://code.google.com/p/pysam/issues/detail?id=105


# import pp

import multiprocessing as mp

import pysam
import argparse

from datetime import datetime
import os, shutil
# import shlex, subprocess
from pynnotator import settings

dbnfsp_header = '''##INFO=<ID=dbNSFP_DANN_score,Number=A,Type=Float,Description="DANN is a functional prediction score retrained based on the training data     of CADD using deep neural network. Scores range from 0 to 1. A larger number indicate a higher probability to be damaging. More information of this score can be found in doi: 10.1093/bioinformatics/btu703.">
##INFO=<ID=dbNSFP_DANN_rankscore,Number=A,Type=Float,Description="DANN scores were ranked among all DANN scores in dbNSFP. The rankscore is the ratio of the rank of the score over the total number of DANN scores in dbNSFP">
##INFO=<ID=dbNSFP_CADD_raw,Number=A,Type=Float,Description="CADD raw score for funtional prediction of a SNP. Please refer to Kircher et al. (2014) Nature Genetics 46(3):310-5 for details. The larger the score the more likely the SNP has damaging effect.">
##INFO=<ID=dbNSFP_CADD_raw_rankscore,Number=A,Type=Float,Description="CADD raw scores were ranked among all CADD raw scores in dbNSFP. The rankscore is the ratio of the rank of the score over the total number of CADD raw scores in dbNSFP">
##INFO=<ID=dbNSFP_CADD_phred,Number=A,Type=Float,Description="CADD phred-like score. This is phred-like rank score based on whole genome CADD raw scores. Please refer to Kircher et al. (2014) Nature Genetics 46(3):310-5 for details. The larger the score the more likely the SNP has damaging effect.">
'''

# cores = int(args.cores)
# prefix = 'cadd_dann'
# if not os.path.exists(prefix):
#     os.makedirs(prefix)

class CADD_DANN_Annotator(object):

    def __init__(self, vcf_file=None, cores=None):
        
        self.vcf_file = vcf_file

        # print('self.resources', self.resources)
        self.cores = int(cores)

        self.filename = os.path.splitext(os.path.basename(str(vcf_file)))[0]
        #create folder validator if it doesn't exists
        if not os.path.exists('cadd_dann'):
            os.makedirs('cadd_dann')

    def run(self):

        tstart = datetime.now()

        print(tstart, 'Starting cadd dann annotator: ', self.vcf_file)
        
        # std = self.annotator()

        self.splitvcf(self.vcf_file)

        pool = mp.Pool()
        pool.map(self.annotate, range(1,self.cores+1))
        # pool.close()
        # pool.join()

        prefix = 'cadd_dann'
        # # Define your jobs
        # jobs = []
        final_parts = []
        for n in range(0,self.cores):
            index = n+1
            final_file = 'cadd_dann/cadd_dann.%s.vcf' % (index)
            final_parts.append(final_file)
        
        command = 'cat %s/header.vcf ' % (prefix) + " ".join(final_parts) + '> %s/cadd_dann.vcf' % (prefix)
        std = os.system(command)

        tend = datetime.now()
        annotation_time =  tend - tstart
        print(tend, 'Finished vcf annotator, it took: ', annotation_time)


    def partition(self, lst, n):
            division = len(lst) / float(n)
            return [ lst[int(round(division * i)): int(round(division * (i + 1)))] for i in range(n) ]

    def splitvcf(self, vcffile):
        # print('split file', vcffile)
        # print 'numero de cores', cores
        prefix = 'cadd_dann'
        vcf_reader = open('%s' % (vcffile))
        header_writer = open('%s/header.vcf' % (prefix), 'w')
        body_writer = open('%s/body.vcf' % (prefix), 'w')
        
        count_lines = 0
        for line in vcf_reader:
            if line.startswith('#'):
                if line.startswith('#CHROM'):
                    header_writer.writelines(dbnfsp_header)
                header_writer.writelines(line)
            else:
                body_writer.writelines(line)
        header_writer.close()
        body_writer.close()
        
        vcf_reader = open('%s/body.vcf' % (prefix))

        groups = self.partition(list(vcf_reader.readlines()), self.cores)
        for c, group in enumerate(groups):
            # print 'group', len(group)
            # print 'c', c
            part = c + 1
            part_writer = open('%s/part.%s.vcf' % (prefix, part), 'w')
            for line in group:
                part_writer.writelines(line)
            part_writer.close()

    #convert and annotate the vcf file to snpeff
    def annotate(self, out_prefix):
        #print 'Hello'
        #print self.dbnfsp_reader
        #header is at:
        # 67      CADD_raw: 
        # 68      CADD_raw_rankscore: 
        # 69      CADD_phred: 
        
        # 70      DANN_score: 
        # 71      DANN_rankscore: 
        dann_start = 69
        dann_end = 71
        cadd_start = 66 
        cadd_end = 69


        # print 'input',vcffile, out_prefix, dbnsfp 
        dbnfsp_reader = pysam.Tabixfile(settings.dbnsfp, 'r')
        
        # print('header')
        for item in dbnfsp_reader.header:
            header = item.decode('utf-8').strip().split('\t')

        # header = dbnfsp_reader.header.next().strip().split('\t')

        vcffile = 'cadd_dann/part.%s.vcf' % (out_prefix)

        vcf_reader = open('%s' % (vcffile))
        vcf_writer = open('cadd_dann/cadd_dann.%s.vcf' % (out_prefix), 'w')
        
        for line in vcf_reader:
            if line.startswith('#'):
                if line.startswith('#CHROM'):
                    vcf_writer.writelines(dbnfsp_header)
                vcf_writer.writelines(line)
            else:
                variant = line.split('\t')
                variant[0] = variant[0].replace('chr', '')
                index = '%s-%s' % (variant[0], variant[1])
                #print index
                try:
                    records = dbnfsp_reader.fetch(variant[0], int(variant[1])-1, int(variant[1]))
                except:
                    records = []
                    
                for record in records:
                    ann = record.strip().split('\t')

                    ispresent = False
                    if variant[3] == ann[2]:
                        alts = variant[4].split(',')
                        alts_ann = ann[3].split(',')
                        #compare ALT
                        for alt in alts:
                            if alt in alts_ann:
                                ispresent = True

                    if ispresent:
                        new_ann = []
                        #this annotated all cadd and dann from dbnfsp2.5
                        # for k, item in enumerate(header[dbnfsp_start:dbnfsp_end]):
                        #     idx = k+dbnfsp_start
                        #     if ann[idx] != '.':
                        #         new_ann.append('dbNSFP_%s=%s' % (item, ann[idx].replace(';', '|')))

                        for k, item in enumerate(header[dann_start:dann_end]):
                            idx = k+dann_start
                            if ann[idx] != '.':
                                new_ann.append('dbNSFP_%s=%s' % (item, ann[idx].replace(';', '|')))
                        for k, item in enumerate(header[cadd_start:cadd_end]):
                            idx = k+cadd_start
                            if ann[idx] != '.':
                                new_ann.append('dbNSFP_%s=%s' % (item, ann[idx].replace(';', '|')))

                        variant[7] = '%s;%s' % (variant[7], ";".join(new_ann))
                vcf_writer.writelines("\t".join(variant))


if  __name__ == '__main__' :
    
    
    parser = argparse.ArgumentParser(description='Annotate a VCF File with DbNSFP.')

    parser.add_argument('-i', dest='vcf_file', required=True, metavar='example.vcf', help='a VCF file to be annotated')

    parser.add_argument('-n', dest='cores', required=True, metavar='4', help='number of cores to use')

    args = parser.parse_args()

    cadd_dann = CADD_DANN_Annotator(args.vcf_file, args.cores)
    cadd_dann.run()



# def annotate(vcffile, index, dbnsfp):
#     print 'vcf', vcffile, index
#     # filename = os.path.splitext(os.path.basename(str(vcffile)))[0]
#     dbnfsp_reader = pysam.Tabixfile(dbnsfp)

# splitvcf(args.vcffile)

# # job_server = pp.Server()
# prefix = 'cadd_dann'
# # Define your jobs
# jobs = []
# final_parts = []
# for n in range(0,cores):
#     index = n+1
#     part = '%s/part.%s.vcf' % (prefix, index)
    
#     job = Process(target=annotate, args=(part, index, args.dbnsfp))
#     final_file = 'cadd_dann/cadd_dann.%s.vcf' % (index)
#     final_parts.append(final_file)
#     jobs.append(job)


# for job in jobs:
#     job.start()

# for job in jobs:
#     job.join()

# command = 'cat %s/header.vcf ' % (prefix) + " ".join(final_parts) + '> %s/cadd_dann.vcf' % (prefix)
# os.system(command)
# #merge all files 



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
from datetime import datetime
import os, shutil
import shlex, subprocess
from subprocess import call

from pynnotator import settings

"""
- Search and Remove genotypes with [0/0, ./.]
- Search and Replace chr from the beggining of the chromossome names
- Sort VCF by 1...22, X, Y, MT
- Remove previous snpeff annotations from VCF
- Remove annotation sumGLbyD from VCF that causes imcompatibility with snpeff
"""

toolname = 'sanity_check'

#enable perl5_lib
env = os.environ.copy()
env['PERL5LIB'] = settings.vcftools_dir_perl


class Sanity_check(object):
    def __init__(self, vcf_file=None):
        
        self.vcf_file = vcf_file
        self.filename = os.path.splitext(os.path.basename(str(vcf_file)))[0]
        
        #create folder sanity_check if it doesn't exists
        if not os.path.exists('sanity_check'):
            os.makedirs('sanity_check')
        #enter inside folder
        # os.chdir('sanity_check')
        
    
    def run(self):

        tstart = datetime.now()
        print(tstart, 'Starting sanity_check: ', self.vcf_file)
        
        self.check()

        tend = datetime.now()
        annotation_time =  tend - tstart
        print(tend, 'Finished sanity_check, it took: ', annotation_time)        

    #sanity vcf file with Vcftools
    def check(self):

        file_vcf = open("%s" % (self.vcf_file), 'r')
        out_vcf = open('sanity_check/onlyvariants.vcf', 'w')
        for line in file_vcf:
            if line.startswith('#'):
                if not line.startswith('##INFO=<ID=EFF'):
                    if not line.startswith('##INFO=<ID=sumGLbyD'):
                        if not line.startswith('##INFO=<ID=CSQ'):
                            out_vcf.writelines(line)
            else:
                row = line.split('\t')

                #removing EFF INFO from VCF
                INFO = row[7]
                NEWINFO = []
                for info in INFO.split(";"):
                        #removing snpeff from annotations
                        if "EFF=" in info:
                            pass
                        #ugly hack to remove this field, because of some bugs in GATK/snpeff that require tyou to have a description about all fields
                        elif "sumGLbyD=" in info:
                            pass
                        elif "CSQ=" in info:
                            pass
                        else:
                            NEWINFO.append(info)
                row[7] = ";".join(NEWINFO)
                row[2] = '.'
                #remove info field from vcf
                # row[7] = ''
                genotype = row[-1].strip().split(':')[0]
                #remove chr from the beggining of chromossome name
                row[0] = row[0].replace('chr', '')
                #remove genotypes with 0/0
                forbidden = ['0/0', './.']
                allowed_chr = ['1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '20', '21', '22', 'X', 'Y', 'M', 'MT']
                if genotype not in forbidden:
                    if row[0] in allowed_chr: 
                        out_vcf.writelines('\t'.join(row))

        out_vcf.close()    
        file_vcf.close()

        #remove EFF Field
        # vcf_tools_dir = '/lgc/programs/vcftools_0.1.10/bin'
        # os.environ["PERL5LIB"] = "/lgc/programs/vcftools_0.1.10/lib/perl5/site_perl/"

        # command = '%s/vcf-annotate onlyvariants.vcf -r INFO/EFF > withouteff.vcf' % (vcf_tools_dir)
        # os.system(command)

        #sort VCF
        #logging.info('Starting Sort VCF')
        #get header

        command = "grep '^#' sanity_check/onlyvariants.vcf > sanity_check/checked.vcf"
        call(command, shell=True)

        #only chromossome numbers first
        
        command = "grep -E -v '^X|^Y|^M|^#|^GL' sanity_check/onlyvariants.vcf | sort -n -k1 -k2 >> sanity_check/checked.vcf"
        call(command, shell=True)

        #only X
        command = "grep -E '^X' sanity_check/onlyvariants.vcf | sort -k1,1d -k2,2n >> sanity_check/checked.vcf"
        call(command, shell=True)

        #only Y
        command = "grep -E '^Y' sanity_check/onlyvariants.vcf | sort -k1,1d -k2,2n >> sanity_check/checked.vcf"
        call(command, shell=True)

        #only MT
        command = "grep -E '^M' sanity_check/onlyvariants.vcf | sort -k1,1d -k2,2n >> sanity_check/checked.vcf"
        call(command, shell=True)

        # p = subprocess.call(command, 
        #     cwd=os.getcwd(), 
        #     env=env, 
        #     shell=True)

        time_end = datetime.now()
        print(time_end, 'This vcf was sucessfully checked with sanity_check')

        # if p == 0:
        #     print 'This vcf was validated by vcf-sanity_check'
        # else:
        #     print 'Sorry this vcf could not be annotated by %s' % (toolname)

        

if  __name__ == '__main__' :
    parser = argparse.ArgumentParser(description='Sanity Check a VCf File')

    parser.add_argument('-i', dest='vcf_file', required=True, metavar='example.vcf', help='a VCF file to be annotated')

    args = parser.parse_args()

    sanity_check = Sanity_check(args.vcf_file)
    sanity_check.run()




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
from datetime import datetime
import os, shutil
import shlex, subprocess

from pynnotator import settings
from collections import OrderedDict

import pysam

toolname = 'merge'


class Merge(object):
    def __init__(self, vcffile=None):
        
        self.vcffile = vcffile

        self.filename = os.path.splitext(os.path.basename(str(vcffile)))[0]
        
        #create folder merge if it doesn't exists
        if not os.path.exists('merge'):
            os.makedirs('merge')
        #enter inside folder
        os.chdir('merge')

        self.annotation_files = OrderedDict()

        pysam.tabix_index('../snpeff/snpeff.output.vcf', preset='vcf')

        self.annotation_files['snpeff'] = {
            'info':'EFF', 
            'file':pysam.Tabixfile('../snpeff/snpeff.output.vcf.gz', 'r' ,encoding="utf-8")
            }

        pysam.tabix_index('../vep/vep.output.sorted.vcf', preset='vcf')

        self.annotation_files['vep'] = {
            'info':'CSQ', 
            'file':pysam.Tabixfile('../vep/vep.output.sorted.vcf.gz', 'r', encoding="utf-8")
            }

        pysam.tabix_index('../snpsift/snpsift.final.vcf', preset='vcf')

        self.annotation_files['vartype'] = {
            'info':'VARTYPE,SNP,MNP,INS,DEL,MIXED,HOM,HET', 
            'file':pysam.Tabixfile('../snpsift/snpsift.final.vcf.gz', 'r', encoding="utf-8")
            }

        pysam.tabix_index('../decipher/hi_predictions.vcf', preset='vcf')

        self.annotation_files['decipher'] = {
            'info':'HI_PREDICTIONS', 
            'file':pysam.Tabixfile('../decipher/hi_predictions.vcf.gz', 'r', encoding="utf-8")
            }

        pysam.tabix_index('../pynnotator/pynnotator.vcf', preset='vcf')

        #genomes1k dbsnp clinvar esp6500 ensembl_phen ensembl_clin
        self.pynnotator_tags = ['genomes1k', 'dbsnp', 'clinvar', 'esp6500', 'ensembl_phen', 'ensembl_clin']

        self.annotation_files['pynnotator'] = {
            'info':'ALL', 
            'file':pysam.Tabixfile('../pynnotator/pynnotator.vcf.gz', 'r', encoding="utf-8")
            }

        pysam.tabix_index('../func_pred/func_pred.vcf', preset='vcf')

        self.annotation_files['dbnfsp'] = {
            'info':'dbNSFP_SIFT_score,dbNSFP_SIFT_converted_rankscore,dbNSFP_SIFT_pred,dbNSFP_Uniprot_acc_Polyphen2,dbNSFP_Uniprot_id_Polyphen2,dbNSFP_Uniprot_aapos_Polyphen2,dbNSFP_Polyphen2_HDIV_score,dbNSFP_Polyphen2_HDIV_rankscore,dbNSFP_Polyphen2_HDIV_pred,dbNSFP_Polyphen2_HVAR_score,dbNSFP_Polyphen2_HVAR_rankscore,dbNSFP_Polyphen2_HVAR_pred,dbNSFP_LRT_score,dbNSFP_LRT_converted_rankscore,dbNSFP_LRT_pred,dbNSFP_LRT_Omega,dbNSFP_MutationTaster_score,dbNSFP_MutationTaster_converted_rankscore,dbNSFP_MutationTaster_pred,dbNSFP_MutationTaster_model,dbNSFP_MutationTaster_AAE,dbNSFP_MutationAssessor_UniprotID,dbNSFP_MutationAssessor_variant,dbNSFP_MutationAssessor_score,dbNSFP_MutationAssessor_rankscore,dbNSFP_MutationAssessor_pred,dbNSFP_FATHMM_score,dbNSFP_FATHMM_converted_rankscore,dbNSFP_FATHMM_pred,dbNSFP_PROVEAN_score,dbNSFP_PROVEAN_converted_rankscore,dbNSFP_PROVEAN_pred,dbNSFP_Transcript_id_VEST3,dbNSFP_Transcript_var_VEST3,dbNSFP_VEST3_score,dbNSFP_VEST3_rankscore,dbNSFP_MetaSVM_score,dbNSFP_MetaSVM_rankscore,dbNSFP_MetaSVM_pred,dbNSFP_MetaLR_score,dbNSFP_MetaLR_rankscore,dbNSFP_MetaLR_pred,dbNSFP_Reliability_index,dbNSFP_CADD_raw,dbNSFP_CADD_raw_rankscore,dbNSFP_CADD_phred,dbNSFP_DANN_score,dbNSFP_DANN_rankscore,dbNSFP_fathmm-MKL_coding_score,dbNSFP_fathmm-MKL_coding_rankscore,dbNSFP_fathmm-MKL_coding_pred,dbNSFP_fathmm-MKL_coding_group,dbNSFP_Eigen-raw,dbNSFP_Eigen-phred,dbNSFP_Eigen-raw_rankscore,dbNSFP_Eigen-PC-raw,dbNSFP_Eigen-PC-raw_rankscore,dbNSFP_GenoCanyon_score,dbNSFP_GenoCanyon_score_rankscore,dbNSFP_integrated_fitCons_score,dbNSFP_integrated_fitCons_rankscore,dbNSFP_integrated_confidence_value,dbNSFP_GM12878_fitCons_score,dbNSFP_GM12878_fitCons_rankscore,dbNSFP_GM12878_confidence_value,dbNSFP_H1-hESC_fitCons_score,dbNSFP_H1-hESC_fitCons_rankscore,dbNSFP_H1-hESC_confidence_value,dbNSFP_HUVEC_fitCons_score,dbNSFP_HUVEC_fitCons_rankscore,dbNSFP_HUVEC_confidence_value,dbNSFP_GERP++_NR,dbNSFP_GERP++_RS,dbNSFP_GERP++_RS_rankscore,dbNSFP_phyloP100way_vertebrate,dbNSFP_phyloP100way_vertebrate_rankscore,dbNSFP_phyloP20way_mammalian,dbNSFP_phyloP20way_mammalian_rankscore,dbNSFP_phastCons100way_vertebrate,dbNSFP_phastCons100way_vertebrate_rankscore,dbNSFP_phastCons20way_mammalian,dbNSFP_phastCons20way_mammalian_rankscore,dbNSFP_SiPhy_29way_pi,dbNSFP_SiPhy_29way_logOdds,dbNSFP_SiPhy_29way_logOdds_rankscore,dbNSFP_clinvar_rs,dbNSFP_clinvar_clnsig,dbNSFP_clinvar_trait,dbNSFP_clinvar_golden_stars', 
            'file':pysam.Tabixfile('../func_pred/func_pred.vcf.gz', 'r', encoding="utf-8")
            }

        self.dbsnp = pysam.Tabixfile(settings.dbsnp, 'r', encoding="utf-8")
    
    def run(self):
        tstart = datetime.now()
        print(tstart, 'Starting merge: ', self.vcffile)
        
        self.merge()

        tend = datetime.now()
        annotation_time =  tend - tstart
        print(tend, 'Finished merge, it took: ', annotation_time)        

    #merge VCF Files with VariantAnnotator
    def merge(self):

        BASE_DIR = os.getcwd()

        #get headers from files
        headers = []
        for annotation in self.annotation_files:
            # print(annotation, self.annotation_files[annotation])
            info = self.annotation_files[annotation]['info'].split(',')
            # print(info)
            ann_vcf_file = self.annotation_files[annotation]['file']

            for line in ann_vcf_file.header:

                line = line.decode('utf-8')

                # print(annotation)
                if annotation == 'pynnotator':
                    for tag in self.pynnotator_tags:
                        string = '##INFO=<ID=%s.' % (tag)
                        if line.startswith(string):
                            headers.append(line)
                else:
                    for tag in info:
                        string = '##INFO=<ID=%s,' % (tag)
                        if line.startswith(string):
                            headers.append(line)
                
        output = open('annotation.final.vcf', 'w')
        input_vcf = open('../sanity_check/checked.vcf')
        for line in input_vcf:
            if line.startswith('#'):
                if line.startswith('#CHROM'):
                    #write extra header
                    for tag in headers:
                        output.writelines(tag+'\n')
                    output.writelines(line)
                else:
                    output.writelines(line)
            else:
                #here annotation comes!
                variant = line.split('\t')
                variant[0] = variant[0].replace('chr', '')
                index = '%s-%s' % (variant[0], variant[1])
                # print(variant)
                annotations = []
                for annotation in self.annotation_files:
                    info = self.annotation_files[annotation]['info'].split(',')
                    # print(info)
                    ann_vcf_file = self.annotation_files[annotation]['file']
                    try:
                        records = ann_vcf_file.fetch(variant[0], int(variant[1])-1, int(variant[1]))
                    except:
                        records = []
                    for record in records:
                        
                        ann = record.split('\t')
                        # print(ann)
                        new_info = ann[7].split(';')

                        if annotation == 'pynnotator':
                            for tag in new_info:
                                string = tag.split('=')[0].split('.')[0]
                                if string in self.pynnotator_tags:
                                    annotations.append(tag)
                        else:
                        # print(info)
                            for tag in new_info:
                                string = tag.split('=')[0]
                                if string in info:
                                    annotations.append(tag)
                        # print(annotations)
                            
                new_info = variant[7]+';'+";".join(annotations)
                # print(new_info)
                variant[7] = new_info

                #now dbsnp
                
                try:
                    records = self.dbsnp.fetch(variant[0], int(variant[1])-1, int(variant[1]))
                except:
                    records = []
                rsids = []
                for record in records:
                    # print(record, variant)
                    row = record.split('\t')
                    rsid = row[2]

                    check_flag = False
                    if variant[3] == row[3]:
                        alts = variant[4].split(',')
                        alts_row = row[4].split(',')
                        #compare ALT
                        for alt in alts:
                            if alt in alts_row:
                                check_flag = True
                    if check_flag:
                        rsids.append(rsid)
                
                if len(rsids) > 0:
                    variant[2] = ",".join(rsids)

                new_line = "\t".join(variant)
                output.writelines(new_line)

                # index()


        output.close()
        #merge 5 vcfs: snpeff, vep, snpsift, hi_index and hgmd 
        # print('merge snpeff')
        #snpeff
        # command = "java -Xmx%s -jar \
        # %s/SnpSift.jar \
        # annotate -info EFF -noId \
        # ../snpeff/snpeff.output.vcf \
        # ../sanity_check/checked.vcf \
        # > snpeff.vcf \
        # "% (settings.snpsift_merge_memory, settings.snpeff_dir)
        # print(command)
        
        # p = subprocess.call(command, 
        #     cwd=os.getcwd(), 
        #     shell=True)
        # # print 'snpeff'
        # # print('merge vep')
        # #vep
        # command = "java -Xmx%s -jar \
        # %s/SnpSift.jar \
        # annotate -info CSQ -noId \
        # ../vep/vep.output.sorted.vcf \
        # snpeff.vcf \
        # > snpeff.vep.vcf \
        # "% (settings.snpsift_merge_memory, settings.snpeff_dir)

        # p = subprocess.call(command, 
        #     cwd=os.getcwd(), 
        #     shell=True)
        # # print('merge snpsift')
        # #snpsift
        # command = "java -Xmx%s -jar \
        # %s/SnpSift.jar \
        # annotate -info VARTYPE,SNP,MNP,INS,DEL,MIXED,HOM,HET -noId \
        # ../snpsift/snpsift.final.vcf \
        # snpeff.vep.vcf \
        # > snpsift.snpeff.vep.vcf \
        # "% (settings.snpsift_merge_memory, settings.snpeff_dir)

        # p = subprocess.call(command, 
        #     cwd=os.getcwd(), 
        #     shell=True)

        # # print('merge hi_index')
        # #hi_index
        # command = "java -Xmx%s -jar \
        # %s/SnpSift.jar \
        # annotate -info HI_INDEX -noId \
        # ../hi_index/hi_index.vcf \
        # snpsift.snpeff.vep.vcf \
        # > hi_index.snpsift.snpeff.vep.vcf \
        # "% (settings.snpsift_merge_memory, settings.snpeff_dir)

        # p = subprocess.call(command, 
        #     cwd=os.getcwd(), 
        #     shell=True)
        
        # # print('merge pynnotator')
        # #pynnotator - 1000genomes, dbsnp, clinvar, esp6500, ensembl
        # command = "java -Xmx%s -jar \
        # %s/SnpSift.jar \
        # annotate -noId \
        # ../pynnotator/pynnotator.vcf \
        # hi_index.snpsift.snpeff.vep.vcf \
        # > 1kgenomes.dbsnp.clinvar.esp.hi_index.snpsift.snpeff.vep.vcf  \
        # " % (settings.snpsift_merge_memory, settings.snpeff_dir)

        # p = subprocess.call(command, 
        #     cwd=os.getcwd(), 
        #     shell=True)

        # # print   'func pred'
        
        # # print('merge cadd_dann')
        # #cadd_dann
        # command = "java -Xmx%s -jar \
        # %s/SnpSift.jar \
        # annotate -noId -info dbNSFP_SIFT_score,dbNSFP_SIFT_converted_rankscore,dbNSFP_SIFT_pred,dbNSFP_Uniprot_acc_Polyphen2,dbNSFP_Uniprot_id_Polyphen2,dbNSFP_Uniprot_aapos_Polyphen2,dbNSFP_Polyphen2_HDIV_score,dbNSFP_Polyphen2_HDIV_rankscore,dbNSFP_Polyphen2_HDIV_pred,dbNSFP_Polyphen2_HVAR_score,dbNSFP_Polyphen2_HVAR_rankscore,dbNSFP_Polyphen2_HVAR_pred,dbNSFP_LRT_score,dbNSFP_LRT_converted_rankscore,dbNSFP_LRT_pred,dbNSFP_LRT_Omega,dbNSFP_MutationTaster_score,dbNSFP_MutationTaster_converted_rankscore,dbNSFP_MutationTaster_pred,dbNSFP_MutationTaster_model,dbNSFP_MutationTaster_AAE,dbNSFP_MutationAssessor_UniprotID,dbNSFP_MutationAssessor_variant,dbNSFP_MutationAssessor_score,dbNSFP_MutationAssessor_rankscore,dbNSFP_MutationAssessor_pred,dbNSFP_FATHMM_score,dbNSFP_FATHMM_converted_rankscore,dbNSFP_FATHMM_pred,dbNSFP_PROVEAN_score,dbNSFP_PROVEAN_converted_rankscore,dbNSFP_PROVEAN_pred,dbNSFP_Transcript_id_VEST3,dbNSFP_Transcript_var_VEST3,dbNSFP_VEST3_score,dbNSFP_VEST3_rankscore,dbNSFP_MetaSVM_score,dbNSFP_MetaSVM_rankscore,dbNSFP_MetaSVM_pred,dbNSFP_MetaLR_score,dbNSFP_MetaLR_rankscore,dbNSFP_MetaLR_pred,dbNSFP_Reliability_index,dbNSFP_CADD_raw,dbNSFP_CADD_raw_rankscore,dbNSFP_CADD_phred,dbNSFP_DANN_score,dbNSFP_DANN_rankscore,dbNSFP_fathmm,dbNSFP_fathmm,dbNSFP_fathmm,dbNSFP_fathmm,dbNSFP_Eigen,dbNSFP_Eigen,dbNSFP_Eigen,dbNSFP_Eigen,dbNSFP_Eigen,dbNSFP_GenoCanyon_score,dbNSFP_GenoCanyon_score_rankscore,dbNSFP_integrated_fitCons_score,dbNSFP_integrated_fitCons_rankscore,dbNSFP_integrated_confidence_value,dbNSFP_GM12878_fitCons_score,dbNSFP_GM12878_fitCons_rankscore,dbNSFP_GM12878_confidence_value,dbNSFP_H1,dbNSFP_H1,dbNSFP_H1,dbNSFP_HUVEC_fitCons_score,dbNSFP_HUVEC_fitCons_rankscore,dbNSFP_HUVEC_confidence_value,dbNSFP_GERP,dbNSFP_GERP,dbNSFP_GERP,dbNSFP_phyloP100way_vertebrate,dbNSFP_phyloP100way_vertebrate_rankscore,dbNSFP_phyloP20way_mammalian,dbNSFP_phyloP20way_mammalian_rankscore,dbNSFP_phastCons100way_vertebrate,dbNSFP_phastCons100way_vertebrate_rankscore,dbNSFP_phastCons20way_mammalian,dbNSFP_phastCons20way_mammalian_rankscore,dbNSFP_SiPhy_29way_pi,dbNSFP_SiPhy_29way_logOdds,dbNSFP_SiPhy_29way_logOdds_rankscore,dbNSFP_clinvar_rs,dbNSFP_clinvar_clnsig,dbNSFP_clinvar_trait,dbNSFP_clinvar_golden_stars \
        # ../func_pred/func_pred.vcf \
        # 1kgenomes.dbsnp.clinvar.esp.hi_index.snpsift.snpeff.vep.vcf \
        # > func_pred.1kgenomes.dbsnp.clinvar.esp.hi_index.snpsift.snpeff.vep.vcf  \
        # "% (settings.snpsift_merge_memory, settings.snpeff_dir)


        # p = subprocess.call(command, 
        #     cwd=os.getcwd(), 
        #     shell=True)

        # # print('merge dbsnp')
        # #dbsnp
        # command = "java -Xmx%s -jar \
        # %s/SnpSift.jar \
        # annotate -id \
        # %s \
        # func_pred.1kgenomes.dbsnp.clinvar.esp.hi_index.snpsift.snpeff.vep.vcf \
        # > annotation.final.vcf  \
        # "% (settings.snpsift_merge_memory, settings.snpeff_dir,settings.dbsnp)


        # p = subprocess.call(command, 
        #     cwd=os.getcwd(), 
        #     shell=True)

        # if p == 0:
        #     print('This vcf was succesfully merged')
        # else:
        #     print('Sorry this vcf could not be merged')


if  __name__ == '__main__' :

    parser = argparse.ArgumentParser(description='MERGE all VCF Files from previous methods.')
    parser.add_argument('-i', dest='vcffile', required=True, metavar='example.vcf', help='a VCF file to be annotated')
    
    args = parser.parse_args()

    merge = Merge(args.vcffile)
    merge.run()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
from datetime import datetime
import os, shutil
import shlex, subprocess

from pynnotator import settings
from subprocess import call

toolname = 'vep'

class Vep(object):
    def __init__(self, vcffile=None):
        
        self.vcffile = vcffile

        self.filename = os.path.splitext(os.path.basename(str(vcffile)))[0]
        
        #create folder vep if it doesn't exists
        if not os.path.exists('vep'):
            os.makedirs('vep')
        #enter inside folder
        # os.chdir('vep')
        
    
    def run(self):
        tstart = datetime.now()
        print(tstart, 'Starting vep annotation: ', self.vcffile)
        
        self.annotate()

        tend = datetime.now()
        annotation_time =  tend - tstart
        print(tend, 'Finished VEP annotation, it took: ', annotation_time)

    #convert and annotate the vcf file to vep
    def annotate(self):

        command = '''perl %s/variant_effect_predictor.pl \
        -i %s \
        --format vcf \
        --dir %s \
        -sift b -polyphen b \
        -o vep/vep.output.vcf --vcf --cache --force_overwrite \
        --no_progress \
        --no_intergenic \
        --numbers \
        --biotype \
        --total_length \
        --coding_only \
        --pick \
        --offline \
        --symbol \
        1>vep/vep.log \
        --fork %s \
        ''' % (settings.vep_dir, self.vcffile, settings.vep_cache_dir, settings.vep_cores)
        # 1> vepreport.log \
        ##--pick \
        # condel_plugin, dbscsnv_plugin, 
        # -plugin %s \
        # --plugin %s \
        
        p = subprocess.call(command, 
            cwd=os.getcwd(), 
            shell=True)

        tend = datetime.now()

        if p == 0:
            print(tend, 'This vcf was sucessfully annotated by %s!' % (toolname))
        else:
            print(tend, 'Sorry this vcf could not be annotated by %s' % (toolname))

        # command = '(grep ^# output.vep.vcf; grep -v ^# output.vep.vcf|sort -k1,1N -k2,2n) > output.vep.sorted.vcf'
        #Sort VCF file 
        

        command = '''grep '^#' vep/vep.output.vcf > vep/vep.output.sorted.vcf'''
        call(command, shell=True)
        
        command = '''grep -E -v '^X|^Y|^M|^#|^GL' vep/vep.output.vcf | sort -n -k1 -k2 >> vep/vep.output.sorted.vcf'''
        call(command, shell=True)

        command = '''grep -E '^X' vep/vep.output.vcf | sort -k1,1d -k2,2n >> vep/vep.output.sorted.vcf'''
        call(command, shell=True)

        command = '''grep -E '^Y' vep/vep.output.vcf | sort -k1,1d -k2,2n >> vep/vep.output.sorted.vcf'''
        call(command, shell=True)

        command = '''grep -E '^M' vep/vep.output.vcf | sort -k1,1d -k2,2n >> vep/vep.output.sorted.vcf'''
        call(command, shell=True)

        tend = datetime.now()
        print(tend, 'Finished sorting vep vcf.')

if  __name__ == '__main__' :
    parser = argparse.ArgumentParser(description='Annotate a VCF File with VEP.')
    parser.add_argument('-i', dest='vcffile', required=True, metavar='example.vcf', help='a VCF file to be annotated')

    args = parser.parse_args()

    vep = Vep(args.vcffile)
    vep.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
from datetime import datetime
import os, shutil
import shlex, subprocess

from pynnotator import settings

class Snpeff(object):
    def __init__(self, vcffile=None):
        
        self.vcffile = vcffile

        self.filename = os.path.splitext(os.path.basename(str(vcffile)))[0]
        
        #create folder snpeff if it doesn't exists
        if not os.path.exists('snpeff'):
            os.makedirs('snpeff')
        #enter inside folder
        # os.chdir('snpeff')
        
    
    def run(self):
        tstart = datetime.now()
        print(tstart, 'Starting snpEff annotation: ', self.vcffile)
        
        self.annotate()

        tend = datetime.now()
        annotation_time =  tend - tstart
        print(tend, 'Finished snpEff annotation, it took: ', annotation_time)        

    #convert and annotate the vcf file to snpeff
    def annotate(self):
        
         
        BASE_DIR = os.getcwd()
        #print BASE_DIR
        #-canon to report only canonical transcript, -o gatk to report only one #GRCh37.64
        #true
        #snpeff 4.0
        command = """java -Xmx%s -jar %s/snpEff.jar eff \
        -c %s/snpEff.config \
        -no-downstream \
        -no-intergenic \
        -no intragenic \
        -onlyProtein \
        -no-intron \
        -no-upstream \
        -noNextProt \
        -no-utr -canon \
        -classic -i vcf -t  %s %s \
        >snpeff/snpeff.output.vcf""" % (settings.snpEff_memory, settings.snpeff_dir, settings.snpeff_dir, settings.snpeff_database, self.vcffile)
        # print(command)
        


        #snpeff 2.0.5d
        # command = "%s/java -Xmx%s -jar %s/snpEff.jar eff \
        # -c %s/snpEff.config \
        # -v -onlyCoding true \
        # -no-downstream -no-intergenic -no-intron -no-upstream -no-utr\
        # -i vcf -o vcf %s %s \
        # >snpeff/snpeff.output.vcf" % (java_path, snpEff_memory, snpeff_dir, snpeff_dir, snpeff_database, self.vcffile)

        # print(command)
        
        # -i vcf 
        # args = shlex.split(command)
        p = subprocess.call(command, 
            cwd=os.getcwd(), 
            shell=True)

        tend = datetime.now()
        if p == 0:
            print(tend, 'This vcf was annotated by snpEff with Success.')
        else:
            print(tend, 'Sorry this vcf could not be anotated by snpeff.')

        return p
        # command = "java -Xmx40G -jar %s/GenomeAnalysisTK.jar \
        # -T VariantAnnotator \
        # -R %s \
        # -A SnpEff \
        # --alwaysAppendDbsnpId \
        # --variant ../%s \
        # --snpEffFile snpEff_output.vcf \
        # -L ../%s \
        # -o snpeff.final.vcf \
        # --dbsnp %s \
        # --log_to_file snpeffgatkreport.log \
        # " % (gatk_dir, reference, self.vcffile, self.vcffile, dbsnp)
        # #        --logging_level ERROR
        # # print 'snpeff command', command

        # p = subprocess.call(command, 
        #     cwd=os.getcwd(), 
        #     shell=True)
        # if p == 0:
        #     print 'Finished running GATK VariantAnnotator after snpEff'
        # else:
        #     print 'Sorry was an error in GATK VariantAnnotator after snpEff'

# 

if  __name__ == '__main__' :
    parser = argparse.ArgumentParser(description='Annotate a VCF File with Snpeff.')

    parser.add_argument('-i', dest='vcffile', required=True, metavar='example.vcf', help='a VCF file to be annotated')

    args = parser.parse_args()

    snpeff = Snpeff(args.vcffile)
    snpeff.run()



from setuptools import setup

setup(name='pynnotator',
      version='0.6',
      description='A Python Annotation Framework for VCFs using multiple tools (Ex. VEP, SnpEff and SnpSift) and databases (Ex. 1000genomes, dbSNP and dbnfsp) .',
      url='http://github.com/raonyguimaraes/pynnotator',
      author='Raony Guimaraes',
      author_email='raony@torchmed.com',
      license='MIT',
      packages=['pynnotator', 'pynnotator.helpers', 'pynnotator.tests'],
      install_requires=[
          'pysam',
      ],
      test_suite='nose.collector',
      tests_require=['nose'],
      scripts=['bin/pynnotator'],
      include_package_data=True,
      zip_safe=False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import os
import sys

import time
from datetime import datetime

import csv
import shutil
import shlex, subprocess

from threading import Thread

from . import settings

from .helpers import validator, sanity_check, snpeff, vep, decipher, snpsift, vcf_annotator, func_pred, merge

class Annotator(object):

    is_validated = False

    def __init__(self, vcf_file=None):
        
        self.filename = os.path.splitext(os.path.basename(str(vcf_file)))[0]

        self.vcf_file = os.path.abspath(vcf_file)

        #this is used to create the folder with the right name
        self.filename = self.filename.replace('.vcf.gz', '').replace('.vcf', '')

        # create a folder for the annotation if it doesn't exists, 
        # or delete and create if the folder already exists
        self.ann_name = "ann_%s" % (self.filename)

        if os.path.exists(self.ann_name):
            shutil.rmtree(self.ann_name)

        os.makedirs(self.ann_name)
        os.chdir(self.ann_name)
        



    def run(self):
        
        tstart = datetime.now()

        if self.vcf_file.endswith('.vcf.gz'):
            path = os.path.dirname(self.vcf_file)
            new_name = '%s/%s/%s.vcf' % (path, self.ann_name, self.filename)
            command='gunzip -c -d %s > %s' % (self.vcf_file, new_name)
            self.shell(command)
            self.vcf_file = new_name
        else:
            path = os.path.dirname(self.vcf_file)
            new_name = '%s/%s/%s.vcf' % (path, self.ann_name, self.filename)
            command='cp %s %s' % (self.vcf_file, new_name)
            self.shell(command)
            self.vcf_file = new_name
        

        #run vcftools vcf-validator on this file 
        validator = Thread(target=self.validator)
        validator.start()
        #wait the process to finish before continue to the next steps 
        validator.join()

        # #Check if thew vcf file is validated
        if(self.is_validated):#
            
            threads = []

            sanitycheck = Thread(target=self.sanitycheck)
            sanitycheck.start()
            # #wait till finish to continue
            sanitycheck.join()

            self.vcf_file = 'sanity_check/checked.vcf'

            snpeff = Thread(target=self.snpeff)
            threads.append(snpeff)

            vep = Thread(target=self.vep)
            threads.append(vep)

            decipher = Thread(target=self.decipher)
            threads.append(decipher)

            snpsift = Thread(target=self.snpsift)
            threads.append(snpsift)

            vcf_annotator = Thread(target=self.vcf_annotator)
            threads.append(vcf_annotator)

            func_pred = Thread(target=self.func_pred) #took 0:17:40.699580
            threads.append(func_pred)

            #execute all tasks in parallel
            for thread in threads:
                thread.start()
                #thread.join()#this option to make it serial
            
            for thread in threads:
                thread.join()

            merge = Thread(target=self.merge)
            merge.start()
            # #wait till finish to continue
            merge.join()
            
            time_end = datetime.now()
            # print(time_end, "Annotation Completed!")
            tend = datetime.now()
            execution_time = tend -  tstart
            # logging.info('Finished Annotation, it took %s' % (execution_time))
            print(time_end, 'Finished Annotation, it took %s' % (execution_time))

            output = """
A       A G       T G       A       A G       T G       A
| C   C | | C   C | | A   C | C   C | | C   C | | A   C |
| | T | | | | A | | | | G | | | T | | | | A | | | | G | |
| G   G | | G   G | | T   G | G   G | | G   G | | T   G |
T       T C       A C       T       T C       A C       T
"""
            print(output)

    def log_message(self, message):
        print(message)
        # logging.info(message)
    def shell(self, command):
        # logging.info('Running Command %s' % (command))
        # logging.info('OS CWD: %s' % (os.getcwd()))
        print('Running Command %s' % (command))
        # print 'OS CWD: %s' % (os.getcwd())
        
        try:
            p = subprocess.check_output(command, 
            cwd=os.getcwd(), 
            shell=True)
            # logging.info('Command Output: %s' % (p))
        except subprocess.CalledProcessError as e:
            print('CalledProcessError:', e)

    #if can validate return True else return False
    def validator(self):
        """
        VCF Validator

        """
        tstart = datetime.now()
        v = validator.Validator(self.vcf_file)
        std = v.run()
        if std == 0:
            self.is_validated = True

        tend = datetime.now()
        execution_time = tend -  tstart
        
    def sanitycheck(self):
        """
        Search and Remove variants with [0/0, ./.]
        Search and Replace chr from the beggining of the chromossomes to get positionning.    
        Sort VCF by 1...22, X, Y, MT and nothing else
        #Discard other variants
        """
        #logging.info('Starting Sanity Check...')
        tstart = datetime.now()

        # command = 'python %s/sanity_check.py -i %s' % (scripts_dir, self.vcf_file)
        # self.shell(command)

        sc = sanity_check.Sanity_check(self.vcf_file)
        std = sc.run()
        
        tend = datetime.now()
        execution_time = tend -  tstart
        #logging.info('Finished Sanity Check, it took %s' % (execution_time))

    def snpeff(self):
        """
        Annotation with snpEff 
        """        
        #calculate time thread took to finish
        #logging.info('Starting snpEff')
        tstart = datetime.now()

        se = snpeff.Snpeff(self.vcf_file)
        std = se.run()

        tend = datetime.now()
        execution_time = tend -  tstart
        #logging.info('Finished snpEff, it took %s' % (execution_time))
        # print(tend, 'Finished snpEff, it took %s' % (execution_time))

        #subprocess.call(args)

    def vep(self):
        """VEP"""
        
        #calculate time thread took to finish
        #logging.info('Starting VEP ')
        tstart = datetime.now()

        vep_obj = vep.Vep(self.vcf_file)
        std = vep_obj.run()
        
        # command = 'python %s/vep.py -i sanity_check/checked.vcf' % (scripts_dir)
        # self.shell(command)

        # command = 'mv vep/vep.log log/'
        # os.system(command)
        
        # command = 'mv vep/vep.output.vcf_summary.html reports/vep_summary.html'
        # os.system(command)

        tend = datetime.now()
        execution_time = tend -  tstart
        #logging.info('Finished VEP, it took %s' % (execution_time))
        # print(tend, 'Finished VEP, it took %s' % (execution_time))
    
   
    def decipher(self):
        """Decipher """
        
        #calculate time thread took to finish
        #logging.info('Starting HI score')
        tstart = datetime.now()
 
        decipher_obj = decipher.Decipher(self.vcf_file)
        decipher_obj.run()

        tend = datetime.now()
        execution_time = tend -  tstart
        #logging.info('Finished HI Score, it took %s' % (execution_time))
        # print(tend, 'Finished Decipher, it took %s' % (execution_time))

    def hgmd(self):
        """Hi Index """

        
        #calculate time thread took to finish
        #logging.info('Starting HI score')
        tstart = datetime.now()
        
        command = 'python %s/hgmd.py -i sanity_check/checked.vcf' % (scripts_dir)
        self.shell(command)


        tend = datetime.now()
        execution_time = tend -  tstart
        #logging.info('Finished HI Score, it took %s' % (execution_time))
        # print('Finished HGMD, it took %s' % (execution_time))
        
   
    def merge(self):

        tstart = datetime.now()
        print(tstart, "Merging all VCF Files...")
        t_merge_start = datetime.now()
        # # #merge VCF Files
        # command = 'python %s/merge.py -i sanity_check/checked.vcf' % (scripts_dir)
        # self.shell(command)
        mg = merge.Merge(self.vcf_file)
        mg.run()

        t_merge_end = datetime.now()
        execution_time = t_merge_end -  t_merge_start

        # logging.info('Finished Merging VCF, it took %s' % (execution_time))
        # print(t_merge_end, 'Finished Merging VCF, it took %s' % (execution_time))
        
        #compress annotation file to save space
        # command = '%s/bgzip merge/annotation.final.vcf' % (tabix_path)
        # os.system(command)
        # print('before merge', os.getcwd())
        #move final file one up and delete folder!
        command = 'mv annotation.final.vcf ../'
        os.system(command)

    def snpsift(self):
        """SnpSift"""
        
        tstart = datetime.now()
        
        # command = 'python %s/snpsift.py -i sanity_check/checked.vcf 2>log/snpsift.log' % (scripts_dir)
        # self.shell(command)

        ss = snpsift.SnpSift(self.vcf_file)
        ss.run()

        
        tend = datetime.now()
        execution_time = tend -  tstart
        #logging.info('Finished annovar, it took %s' % (execution_time))
        # print(tend, 'Finished snpsift, it took %s' % (execution_time))

    def vcf_annotator(self):
        """Vcf annotator"""
        
        tstart = datetime.now()

        #python ../scripts/annotate_vcfs.py -i mm13173_14.ug.target1.vcf -r 1000genomes dbsnp138 clinvar esp6500 -a ../data/1000genomes/ALL.wgs.integrated_phase1_v3.20101123.snps_indels_sv.sites.vcf.gz ../data/dbsnp138/00-All.vcf.gz ../data/dbsnp138/clinvar_00-latest.vcf.gz ../data/ESP6500/ESP6500.vcf.gz

        # command = 'python %s/vcf_annotator_parallel.py -n %s -i sanity_check/checked.vcf -r 1000genomes dbsnp clinvar esp6500 -a %s %s %s %s 2>log/pynnotator.log' % (scripts_dir, pynnotator_cores, genomes1k, dbsnp, clinvar, esp)
        # self.shell(command)

        resources = "genomes1k dbsnp clinvar esp6500 ensembl_phen ensembl_clin"#    
        resources = resources.split(' ')
        annfiles = [
        "%s/1000genomes/%s" % (settings.data_dir, settings.genomes1k_file),
        "%s/dbsnp/%s" % (settings.data_dir, settings.dbsnp_file),
        "%s/dbsnp/%s" % (settings.data_dir, settings.clinvar_file),
        "%s/esp6500/%s" % (settings.data_dir, settings.esp_final_file),
        "%s/ensembl/%s" % (settings.data_dir, settings.ensembl_phenotype_file),
        "%s/ensembl/%s" % (settings.data_dir, settings.ensembl_clinically_file),
        ]
        #
 


        # annfiles = " ".join(annfiles)

        # annfiles = ["%s/1000genomes/%s" % (settings.data_dir, settings.genomes1k_file)]

        annotator_obj =  vcf_annotator.VCF_Annotator(self.vcf_file, annfiles, resources, settings.vcf_annotator_cores)

        annotator_obj.run()
        
        tend = datetime.now()
        execution_time = tend -  tstart
        #logging.info('Finished annovar, it took %s' % (execution_time))
        # print(tend, 'Finished vcf_annotator, it took %s' % (execution_time))
    def func_pred(self):
        """func_pred"""
        
        tstart = datetime.now()

        #python ../scripts/annotate_vcfs.py -i mm13173_14.ug.target1.vcf -r 1000genomes dbsnp138 clinvar esp6500 -a ../data/1000genomes/ALL.wgs.integrated_phase1_v3.20101123.snps_indels_sv.sites.vcf.gz ../data/dbsnp138/00-All.vcf.gz ../data/dbsnp138/clinvar_00-latest.vcf.gz ../data/ESP6500/ESP6500.vcf.gz

        # command = 'python %s/cadd_dann.py -n %s -i sanity_check/checked.vcf 2>log/cadd_dann.log' % (scripts_dir, cadd_vest_cores)
        # self.shell(command)
        fp = func_pred.FUNC_PRED_Annotator(self.vcf_file, settings.func_pred_cores)
        fp.run()
        
        tend = datetime.now()
        execution_time = tend -  tstart
        #logging.info('Finished annovar, it took %s' % (execution_time))
        # print(tend, 'Finished Func Pred, it took %s' % (execution_time))



if __name__=="__main__":
    
    parser = argparse.ArgumentParser(description='Annotate a VCF File with Annotator.')
    parser.add_argument('-i', dest='vcf_file', required=True, metavar='example.vcf', help='a VCF file to be annotated')
    args = parser.parse_args()

    a = Annotator(args.vcf_file)
    a.run()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#parallel https://code.google.com/p/pysam/issues/detail?id=105


# import pp

import multiprocessing as mp

import pysam
import argparse

from datetime import datetime
import os, shutil
# import shlex, subprocess
from pynnotator import settings

dbnfsp_header = '''##INFO=<ID=SIFT_score,Number=A,Type=String,Description="SIFT score (SIFTori). Scores range from 0 to 1. The smaller the score the more likely the SNP has damaging effect.  Multiple scores separated by ";", corresponding to Ensembl_proteinid.">
##INFO=<ID=SIFT_converted_rankscore,Number=A,Type=String,Description="SIFTori scores were first converted to SIFTnew=1-SIFTori, then ranked among all SIFTnew scores in dbNSFP. The rankscore is the ratio of  the rank the SIFTnew score over the total number of SIFTnew scores in dbNSFP.  If there are multiple scores, only the most damaging (largest) rankscore is presented. The rankscores range from 0.00963 to 0.91219.">
##INFO=<ID=SIFT_pred,Number=A,Type=String,Description="If SIFTori is smaller than 0.05 (rankscore>0.395) the corresponding nsSNV is predicted as \"D(amaging)\"; otherwise it is predicted as "T(olerated)".  Multiple predictions separated by \";\" 
##INFO=<ID=SIFT_pred,Number=A,Type=String,Description="Uniprot_acc_Polyphen2: Uniprot accession number provided by Polyphen2. Multiple entries separated by \";\".">
##INFO=<ID=Uniprot_id_Polyphen2,Number=A,Type=String,Description="Uniprot ID numbers corresponding to Uniprot_acc_Polyphen2. Multiple entries separated by \";\".">
##INFO=<ID=Uniprot_aapos_Polyphen2,Number=A,Type=String,Description="amino acid position as to Uniprot_acc_Polyphen2. Multiple entries separated by \";\".">
##INFO=<ID=Polyphen2_HDIV_score,Number=A,Type=String,Description="Polyphen2 score based on HumDiv, i.e. hdiv_prob. The score ranges from 0 to 1.  Multiple entries separated by ";", corresponding to Uniprot_acc_Polyphen2.">
##INFO=<ID=Polyphen2_HDIV_rankscore,Number=A,Type=String,Description="Polyphen2 HDIV scores were first ranked among all HDIV scores in dbNSFP. The rankscore is the ratio of the rank the score over the total number of  the scores in dbNSFP. If there are multiple scores, only the most damaging (largest)  rankscore is presented. The scores range from 0.02634 to 0.89865.">
##INFO=<ID=Polyphen2_HDIV_pred,Number=A,Type=String,Description="Polyphen2 prediction based on HumDiv, "D" ("probably damaging", HDIV score in [0.957,1] or rankscore in [0.52844,0.89865]), "P" ("possibly damaging",  HDIV score in [0.453,0.956] or rankscore in [0.34282,0.52689]) and "B" ("benign",  HDIV score in [0,0.452] or rankscore in [0.02634,0.34268]). Score cutoff for binary  classification is 0.5 for HDIV score or 0.3528 for rankscore, i.e. the prediction is  "neutral" if the HDIV score is smaller than 0.5 (rankscore is smaller than 0.3528),  and "deleterious" if the HDIV score is larger than 0.5 (rankscore is larger than  0.3528). Multiple entries are separated by ";".">
##INFO=<ID=Polyphen2_HVAR_score,Number=A,Type=String,Description="Polyphen2 score based on HumVar, i.e. hvar_prob. The score ranges from 0 to 1.  Multiple entries separated by ";", corresponding to Uniprot_acc_Polyphen2.">
##INFO=<ID=Polyphen2_HVAR_rankscore,Number=A,Type=String,Description="Polyphen2 HVAR scores were first ranked among all HVAR scores in dbNSFP. The rankscore is the ratio of the rank the score over the total number of  the scores in dbNSFP. If there are multiple scores, only the most damaging (largest)  rankscore is presented. The scores range from 0.01257 to 0.97092.">
##INFO=<ID=Polyphen2_HVAR_pred,Number=A,Type=String,Description="Polyphen2 prediction based on HumVar, "D" ("probably damaging", HVAR score in [0.909,1] or rankscore in [0.62797,0.97092]), "P" ("possibly damaging",  HVAR in [0.447,0.908] or rankscore in [0.44195,0.62727]) and "B" ("benign", HVAR  score in [0,0.446] or rankscore in [0.01257,0.44151]). Score cutoff for binary  classification is 0.5 for HVAR score or 0.45833 for rankscore, i.e. the prediction  is "neutral" if the HVAR score is smaller than 0.5 (rankscore is smaller than  0.45833), and "deleterious" if the HVAR score is larger than 0.5 (rankscore is larger  than 0.45833). Multiple entries are separated by ";". LRT_score: The original LRT two-sided p-value (LRTori), ranges from 0 to 1.">
##INFO=<ID=LRT_converted_rankscore,Number=A,Type=String,Description="LRTori scores were first converted as LRTnew=1-LRTori*0.5 if Omega<1, or LRTnew=LRTori*0.5 if Omega>=1. Then LRTnew scores were ranked among all  LRTnew scores in dbNSFP. The rankscore is the ratio of the rank over the total number  of the scores in dbNSFP. The scores range from 0.00162 to 0.84324.">
##INFO=<ID=LRT_pred,Number=A,Type=String,Description="LRT prediction, D(eleterious), N(eutral) or U(nknown), which is not solely determined by the score.  LRT_Omega: estimated nonsynonymous-to-synonymous-rate ratio (Omega, reported by LRT)">
##INFO=<ID=MutationTaster_score,Number=A,Type=String,Description="MutationTaster p-value (MTori), ranges from 0 to 1.  Multiple scores are separated by ";". Information on corresponding transcript(s) can  be found by querying http://www.mutationtaster.org/ChrPos.html">
##INFO=<ID=MutationTaster_converted_rankscore,Number=A,Type=String,Description="The MTori scores were first converted: if the prediction is "A" or "D" MTnew=MTori; if the prediction is "N" or "P", MTnew=1-MTori. Then MTnew  scores were ranked among all MTnew scores in dbNSFP. If there are multiple scores of a  SNV, only the largest MTnew was used in ranking. The rankscore is the ratio of the rank of the score over the total number of MTnew scores in dbNSFP. The scores range from 0.08979 to 0.81033.">
##INFO=<ID=MutationTaster_pred,Number=A,Type=String,Description="MutationTaster prediction, "A" ("disease_causing_automatic"), "D" ("disease_causing"), "N" ("polymorphism") or "P" ("polymorphism_automatic"). The  score cutoff between "D" and "N" is 0.5 for MTnew and 0.31713 for the rankscore. MutationTaster_model: MutationTaster prediction models. MutationTaster_AAE: MutationTaster predicted amino acid change. MutationAssessor_UniprotID: Uniprot ID number provided by MutationAssessor. MutationAssessor_variant: AA variant as to MutationAssessor_UniprotID.">
##INFO=<ID=MutationAssessor_score,Number=A,Type=String,Description="MutationAssessor functional impact combined score (MAori). The score ranges from -5.135 to 6.49 in dbNSFP. ">
##INFO=<ID=MutationAssessor_rankscore,Number=A,Type=String,Description="MAori scores were ranked among all MAori scores in dbNSFP. The rankscore is the ratio of the rank of the score over the total number of MAori  scores in dbNSFP. The scores range from 0 to 1.">
##INFO=<ID=MutationAssessor_pred,Number=A,Type=String,Description="MutationAssessor's functional impact of a variant : predicted functional, i.e. high ("H") or medium ("M"), or predicted non-functional, i.e. low ("L") or neutral ("N"). The MAori score cutoffs between "H" and "M",  "M" and "L", and "L" and "N", are 3.5, 1.935 and 0.8, respectively. The rankscore cutoffs  between "H" and "M", "M" and "L", and "L" and "N", are 0.92922, 0.51944 and 0.19719,  respectively.">
##INFO=<ID=FATHMM_score,Number=A,Type=String,Description="FATHMM default score (weighted for human inherited-disease mutations with Disease Ontology) (FATHMMori). Scores range from -16.13 to 10.64. The smaller the score  the more likely the SNP has damaging effect. Multiple scores separated by ";", corresponding to Ensembl_proteinid.">
##INFO=<ID=FATHMM_converted_rankscore,Number=A,Type=String,Description="FATHMMori scores were first converted to FATHMMnew=1-(FATHMMori+16.13)/26.77, then ranked among all FATHMMnew scores in dbNSFP.  The rankscore is the ratio of the rank of the score over the total number of FATHMMnew  scores in dbNSFP. If there are multiple scores, only the most damaging (largest)  rankscore is presented. The scores range from 0 to 1.">
##INFO=<ID=FATHMM_pred,Number=A,Type=String,Description="If a FATHMMori score is <=-1.5 (or rankscore >=0.81332) the corresponding nsSNV is predicted as "D(AMAGING)"; otherwise it is predicted as "T(OLERATED)". Multiple predictions separated by ";", corresponding to Ensembl_proteinid.">
##INFO=<ID=PROVEAN_score,Number=A,Type=String,Description="PROVEAN score (PROVEANori). Scores range from -14 to 14. The smaller the score the more likely the SNP has damaging effect.  Multiple scores separated by ";", corresponding to Ensembl_proteinid.">
##INFO=<ID=PROVEAN_converted_rankscore,Number=A,Type=String,Description="PROVEANori were first converted to PROVEANnew=1-(PROVEANori+14)/28, then ranked among all PROVEANnew scores in dbNSFP. The rankscore is the ratio of  the rank the PROVEANnew score over the total number of PROVEANnew scores in dbNSFP.  If there are multiple scores, only the most damaging (largest) rankscore is presented. The scores range from 0 to 1.">
##INFO=<ID=PROVEAN_pred,Number=A,Type=String,Description="If PROVEANori <= -2.5 (rankscore>=0.543) the corresponding nsSNV is predicted as "D(amaging)"; otherwise it is predicted as "N(eutral)".  Multiple predictions separated by ";", corresponding to Ensembl_proteinid. Transcript_id_VEST3: Transcript id provided by VEST3. Transcript_var_VEST3: amino acid change as to Transcript_id_VEST3.">
##INFO=<ID=VEST3_score,Number=A,Type=String,Description="VEST 3.0 score. Score ranges from 0 to 1. The larger the score the more likely the mutation may cause functional change.  Multiple scores separated by ";", corresponding to Transcript_id_VEST3. Please note this score is free for non-commercial use. For more details please refer to  http://wiki.chasmsoftware.org/index.php/SoftwareLicense. Commercial users should contact  the Johns Hopkins Technology Transfer office.">
##INFO=<ID=VEST3_rankscore,Number=A,Type=String,Description="VEST3 scores were ranked among all VEST3 scores in dbNSFP. The rankscore is the ratio of the rank of the score over the total number of VEST3  scores in dbNSFP. In case there are multiple scores for the same variant, the largest  score (most damaging) is presented. The scores range from 0 to 1.  Please note VEST score is free for non-commercial use. For more details please refer to  http://wiki.chasmsoftware.org/index.php/SoftwareLicense. Commercial users should contact  the Johns Hopkins Technology Transfer office.">
##INFO=<ID=MetaSVM_score,Number=A,Type=String,Description="Our support vector machine (SVM) based ensemble prediction score, which incorporated 10 scores (SIFT, PolyPhen-2 HDIV, PolyPhen-2 HVAR, GERP++, MutationTaster,  Mutation Assessor, FATHMM, LRT, SiPhy, PhyloP) and the maximum frequency observed in  the 1000 genomes populations. Larger value means the SNV is more likely to be damaging.  Scores range from -2 to 3 in dbNSFP.">
##INFO=<ID=MetaSVM_rankscore,Number=A,Type=String,Description="MetaSVM scores were ranked among all MetaSVM scores in dbNSFP. The rankscore is the ratio of the rank of the score over the total number of MetaSVM  scores in dbNSFP. The scores range from 0 to 1.">
##INFO=<ID=MetaSVM_pred,Number=A,Type=String,Description="Prediction of our SVM based ensemble prediction score,"T(olerated)" or "D(amaging)". The score cutoff between "D" and "T" is 0. The rankscore cutoff between "D" and "T" is 0.82268.">
##INFO=<ID=MetaLR_score,Number=A,Type=String,Description="Our logistic regression (LR) based ensemble prediction score, which incorporated 10 scores (SIFT, PolyPhen-2 HDIV, PolyPhen-2 HVAR, GERP++, MutationTaster,  Mutation Assessor, FATHMM, LRT, SiPhy, PhyloP) and the maximum frequency observed in  the 1000 genomes populations. Larger value means the SNV is more likely to be damaging.  Scores range from 0 to 1.">
##INFO=<ID=MetaLR_rankscore,Number=A,Type=String,Description="MetaLR scores were ranked among all MetaLR scores in dbNSFP. The rankscore is the ratio of the rank of the score over the total number of MetaLR scores in dbNSFP.  The scores range from 0 to 1.">
##INFO=<ID=MetaLR_pred,Number=A,Type=String,Description="Prediction of our MetaLR based ensemble prediction score,"T(olerated)" or "D(amaging)". The score cutoff between "D" and "T" is 0.5. The rankscore cutoff between  "D" and "T" is 0.81113.">
##INFO=<ID=Reliability_index,Number=A,Type=String,Description="Number of observed component scores (except the maximum frequency in the 1000 genomes populations) for MetaSVM and MetaLR. Ranges from 1 to 10. As MetaSVM  and MetaLR scores are calculated based on imputed data, the less missing component  scores, the higher the reliability of the scores and predictions. ">
##INFO=<ID=M,Number=A,Type=String,Description="AP_score: M-CAP score (details in DOI: 10.1038/ng.3703). Scores range from 0 to 1. The larger the score the more likely the SNP has damaging effect. ">
##INFO=<ID=M,Number=A,Type=String,Description="AP_rankscore: M-CAP scores were ranked among all M-CAP scores in dbNSFP. The rankscore is the ratio of the rank of the score over the total number of M-CAP scores in dbNSFP.">
##INFO=<ID=M,Number=A,Type=String,Description="AP_pred: Prediction of M-CAP score based on the authors' recommendation, "T(olerated)" or "D(amaging)". The score cutoff between "D" and "T" is 0.025.">
##INFO=<ID=REVEL_score,Number=A,Type=String,Description="REVEL is an ensemble score based on 13 individual scores for predicting the pathogenicity of missense variants. Scores range from 0 to 1. The larger the score the more  likely the SNP has damaging effect. "REVEL scores are freely available for non-commercial use.   For other uses, please contact Weiva Sieh" (weiva.sieh@mssm.edu)">
##INFO=<ID=REVEL_rankscore,Number=A,Type=String,Description="REVEL scores were ranked among all REVEL scores in dbNSFP. The rankscore is the ratio of the rank of the score over the total number of REVEL scores in dbNSFP.">
##INFO=<ID=MutPred_score,Number=A,Type=String,Description="General MutPred score. Scores range from 0 to 1. The larger the score the more likely the SNP has damaging effect.">
##INFO=<ID=MutPred_rankscore,Number=A,Type=String,Description="MutPred scores were ranked among all MutPred scores in dbNSFP. The rankscore is the ratio of the rank of the score over the total number of MutPred scores in dbNSFP. MutPred_protID: UniProt accession or Ensembl transcript ID used for MutPred_score calculation. MutPred_AAchange: Amino acid change used for MutPred_score calculation.">
##INFO=<ID=MutPred_Top5features,Number=A,Type=String,Description="Top 5 features (molecular mechanisms of disease) as predicted by MutPred with p values. MutPred_score > 0.5 and p < 0.05 are referred to as actionable hypotheses. MutPred_score > 0.75 and p < 0.05 are referred to as confident hypotheses. MutPred_score > 0.75 and p < 0.01 are referred to as very confident hypotheses.">
##INFO=<ID=CADD_raw,Number=A,Type=String,Description="CADD raw score for functional prediction of a SNP. Please refer to Kircher et al. (2014) Nature Genetics 46(3):310-5 for details. The larger the score the more likely the SNP has damaging effect. Scores range from -7.535037 to 35.788538 in dbNSFP.  Please note the following copyright statement for CADD:  "CADD scores (http://cadd.gs.washington.edu/) are Copyright 2013 University of  Washington and Hudson-Alpha Institute for Biotechnology (all rights reserved) but are  freely available for all academic, non-commercial applications. For commercial  licensing information contact Jennifer McCullar (mccullaj@uw.edu)."">
##INFO=<ID=CADD_raw_rankscore,Number=A,Type=String,Description="CADD raw scores were ranked among all CADD raw scores in dbNSFP. The rankscore is the ratio of the rank of the score over the total number of CADD  raw scores in dbNSFP. Please note the following copyright statement for CADD: "CADD  scores (http://cadd.gs.washington.edu/) are Copyright 2013 University of Washington  and Hudson-Alpha Institute for Biotechnology (all rights reserved) but are freely  available for all academic, non-commercial applications. For commercial licensing  information contact Jennifer McCullar (mccullaj@uw.edu)."">
##INFO=<ID=CADD_phred,Number=A,Type=String,Description="CADD phred-like score. This is phred-like rank score based on whole genome CADD raw scores. Please refer to Kircher et al. (2014) Nature Genetics 46(3):310-5  for details. The larger the score the more likely the SNP has damaging effect.  Please note the following copyright statement for CADD: "CADD scores  (http://cadd.gs.washington.edu/) are Copyright 2013 University of Washington and  Hudson-Alpha Institute for Biotechnology (all rights reserved) but are freely  available for all academic, non-commercial applications. For commercial licensing  information contact Jennifer McCullar (mccullaj@uw.edu)."">
##INFO=<ID=DANN_score,Number=A,Type=String,Description="DANN is a functional prediction score retrained based on the training data of CADD using deep neural network. Scores range from 0 to 1. A larger number indicate  a higher probability to be damaging. More information of this score can be found in doi: 10.1093/bioinformatics/btu703. For commercial application of DANN, please contact  Daniel Quang (dxquang@uci.edu)">
##INFO=<ID=DANN_rankscore,Number=A,Type=String,Description="DANN scores were ranked among all DANN scores in dbNSFP. The rankscore is the ratio of the rank of the score over the total number of DANN scores in dbNSFP.">
##INFO=<ID=fathmm,Number=A,Type=String,Description="KL_coding_score: fathmm-MKL p-values. Scores range from 0 to 1. SNVs with scores >0.5 are predicted to be deleterious, and those <0.5 are predicted to be neutral or benign.  Scores close to 0 or 1 are with the highest-confidence. Coding scores are trained using 10 groups of features. More details of the score can be found in  doi: 10.1093/bioinformatics/btv009.">
##INFO=<ID=fathmm,Number=A,Type=String,Description="KL_coding_rankscore: fathmm-MKL coding scores were ranked among all fathmm-MKL coding scores in dbNSFP. The rankscore is the ratio of the rank of the score over the total number  of fathmm-MKL coding scores in dbNSFP.">
##INFO=<ID=fathmm,Number=A,Type=String,Description="KL_coding_pred: If a fathmm-MKL_coding_score is >0.5 (or rankscore >0.28317)  the corresponding nsSNV is predicted as "D(AMAGING)"; otherwise it is predicted as "N(EUTRAL)".">
##INFO=<ID=fathmm,Number=A,Type=String,Description="KL_coding_group: the groups of features (labeled A-J) used to obtained the score. More details can be found in doi: 10.1093/bioinformatics/btv009.">
##INFO=<ID=Eigen_coding_or_noncoding,Number=A,Type=String,Description="Whether Eigen-raw and Eigen-phred scores are based on coding model or noncoding model.">
##INFO=<ID=Eigen,Number=A,Type=String,Description="aw: Eigen score for coding SNVs. A functional prediction score based on conservation, allele frequencies, and deleteriousness prediction using an unsupervised learning method  (doi: 10.1038/ng.3477).  Eigen-phred: Eigen score in phred scale.">
##INFO=<ID=Eigen,Number=A,Type=String,Description="C-raw: Eigen PC score for genome-wide SNVs. A functional prediction score based on conservation, allele frequencies, deleteriousness prediction (for missense SNVs) and epigenomic signals (for synonymous and non-coding SNVs) using an unsupervised learning  method (doi: 10.1038/ng.3477).  Eigen-PC-phred: Eigen PC score in phred scale.">
##INFO=<ID=Eigen,Number=A,Type=String,Description="C-raw_rankscore: Eigen-PC-raw scores were ranked among all Eigen-PC-raw scores in dbNSFP. The rankscore is the ratio of the rank of the score over the total number  of Eigen-PC-raw scores in dbNSFP.">
##INFO=<ID=GenoCanyon_score,Number=A,Type=String,Description="A functional prediction score based on conservation and biochemical annotations using an unsupervised statistical learning. (doi:10.1038/srep10576)">
##INFO=<ID=GenoCanyon_score_rankscore,Number=A,Type=String,Description="GenoCanyon_score scores were ranked among all integrated fitCons scores in dbNSFP. The rankscore is the ratio of the rank of the score over the total number  of GenoCanyon_score scores in dbNSFP.">
##INFO=<ID=integrated_fitCons_score,Number=A,Type=String,Description="fitCons score predicts the fraction of genomic positions belonging to a specific function class (defined by epigenomic "fingerprint") that are under selective  pressure. Scores range from 0 to 1, with a larger score indicating a higher proportion of  nucleic sites of the functional class the genomic position belong to are under selective  pressure, therefore more likely to be functional important. Integrated (i6) scores are integrated across three cell types (GM12878, H1-hESC and HUVEC). More details can be found in doi:10.1038/ng.3196.">
##INFO=<ID=integrated_fitCons_rankscore,Number=A,Type=String,Description="integrated fitCons scores were ranked among all integrated fitCons scores in dbNSFP. The rankscore is the ratio of the rank of the score over the total number  of integrated fitCons scores in dbNSFP.">
##INFO=<ID=integrated_confidence_value,Number=A,Type=String,Description="0 - highly significant scores (approx. p<.003); 1 - significant scores (approx. p<.05); 2 - informative scores (approx. p<.25); 3 - other scores (approx. p>=.25).">
##INFO=<ID=GM12878_fitCons_score,Number=A,Type=String,Description="fitCons score predicts the fraction of genomic positions belonging to a specific function class (defined by epigenomic "fingerprint") that are under selective  pressure. Scores range from 0 to 1, with a larger score indicating a higher proportion of  nucleic sites of the functional class the genomic position belong to are under selective  pressure, therefore more likely to be functional important. GM12878 fitCons scores are based on cell type GM12878. More details can be found in doi:10.1038/ng.3196.">
##INFO=<ID=GM12878_fitCons_rankscore,Number=A,Type=String,Description="GM12878 fitCons scores were ranked among all GM12878 fitCons scores in dbNSFP. The rankscore is the ratio of the rank of the score over the total number  of GM12878 fitCons scores in dbNSFP.">
##INFO=<ID=GM12878_confidence_value,Number=A,Type=String,Description="0 - highly significant scores (approx. p<.003); 1 - significant scores (approx. p<.05); 2 - informative scores (approx. p<.25); 3 - other scores (approx. p>=.25).">
##INFO=<ID=H1,Number=A,Type=String,Description="ESC_fitCons_score: fitCons score predicts the fraction of genomic positions belonging to a specific function class (defined by epigenomic "fingerprint") that are under selective  pressure. Scores range from 0 to 1, with a larger score indicating a higher proportion of  nucleic sites of the functional class the genomic position belong to are under selective  pressure, therefore more likely to be functional important. GM12878 fitCons scores are based on cell type H1-hESC. More details can be found in doi:10.1038/ng.3196.">
##INFO=<ID=H1,Number=A,Type=String,Description="ESC_fitCons_rankscore: H1-hESC fitCons scores were ranked among all H1-hESC fitCons scores in dbNSFP. The rankscore is the ratio of the rank of the score over the total number  of H1-hESC fitCons scores in dbNSFP.">
##INFO=<ID=H1,Number=A,Type=String,Description="ESC_confidence_value: 0 - highly significant scores (approx. p<.003); 1 - significant scores (approx. p<.05); 2 - informative scores (approx. p<.25); 3 - other scores (approx. p>=.25).">
##INFO=<ID=HUVEC_fitCons_score,Number=A,Type=String,Description="fitCons score predicts the fraction of genomic positions belonging to a specific function class (defined by epigenomic "fingerprint") that are under selective  pressure. Scores range from 0 to 1, with a larger score indicating a higher proportion of  nucleic sites of the functional class the genomic position belong to are under selective  pressure, therefore more likely to be functional important. GM12878 fitCons scores are based on cell type HUVEC. More details can be found in doi:10.1038/ng.3196.">
##INFO=<ID=HUVEC_fitCons_rankscore,Number=A,Type=String,Description="HUVEC fitCons scores were ranked among all HUVEC fitCons scores in dbNSFP. The rankscore is the ratio of the rank of the score over the total number  of HUVEC fitCons scores in dbNSFP.">
##INFO=<ID=HUVEC_confidence_value,Number=A,Type=String,Description="0 - highly significant scores (approx. p<.003); 1 - significant scores (approx. p<.05); 2 - informative scores (approx. p<.25); 3 - other scores (approx. p>=.25).">
##INFO=<ID=clinvar_rs,Number=A,Type=String,Description="rs number from the clinvar data set">
##INFO=<ID=clinvar_clnsig,Number=A,Type=String,Description="clinical significance as to the clinvar data set. 0 - unknown, 1 - untested, 2 - Benign, 3 - Likely benign, 4 - Likely pathogenic, 5 - Pathogenic, 6 - drug response, 7 - histocompatibility. A negative score means the the score is for the ref allele">
##INFO=<ID=clinvar_trait,Number=A,Type=String,Description="the trait/disease the clinvar_clnsig referring to">
##INFO=<ID=clinvar_golden_stars,Number=A,Type=String,Description="ClinVar Review Status summary. 0 - no assertion criteria provided, 1 - criteria provided, single submitter, 2 - criteria provided, multiple submitters, no conflicts, 3 - reviewed by expert panel, 4 - practice guideline">
'''

# cores = int(args.cores)
# prefix = 'func_pred'
# if not os.path.exists(prefix):
#     os.makedirs(prefix)

class FUNC_PRED_Annotator(object):

    def __init__(self, vcf_file=None, cores=None):
        
        self.vcf_file = vcf_file

        # print('self.resources', self.resources)
        self.cores = int(cores)

        self.filename = os.path.splitext(os.path.basename(str(vcf_file)))[0]
        #create folder validator if it doesn't exists
        if not os.path.exists('func_pred'):
            os.makedirs('func_pred')

    def run(self):

        tstart = datetime.now()

        print(tstart, 'Starting func pred annotator: ', self.vcf_file)
        
        # std = self.annotator()

        self.splitvcf(self.vcf_file)

        pool = mp.Pool()
        pool.map(self.annotate, range(1,self.cores+1))
        # pool.close()
        # pool.join()

        prefix = 'func_pred'
        # # Define your jobs
        # jobs = []
        final_parts = []
        for n in range(0,self.cores):
            index = n+1
            final_file = 'func_pred/func_pred.%s.vcf' % (index)
            final_parts.append(final_file)
        
        command = 'cat %s/header.vcf ' % (prefix) + " ".join(final_parts) + '> %s/func_pred.vcf' % (prefix)
        std = os.system(command)

        tend = datetime.now()
        annotation_time =  tend - tstart
        print(tend, 'Finished func pred, it took: ', annotation_time)


    def partition(self, lst, n):
            division = len(lst) / float(n)
            return [ lst[int(round(division * i)): int(round(division * (i + 1)))] for i in range(n) ]

    def splitvcf(self, vcffile):
        # print('split file', vcffile)
        # print 'numero de cores', cores
        prefix = 'func_pred'
        vcf_reader = open('%s' % (vcffile))
        header_writer = open('%s/header.vcf' % (prefix), 'w')
        body_writer = open('%s/body.vcf' % (prefix), 'w')
        
        count_lines = 0
        for line in vcf_reader:
            if line.startswith('#'):
                if line.startswith('#CHROM'):
                    header_writer.writelines(dbnfsp_header)
                header_writer.writelines(line)
            else:
                body_writer.writelines(line)
        header_writer.close()
        body_writer.close()
        
        vcf_reader = open('%s/body.vcf' % (prefix))

        groups = self.partition(list(vcf_reader.readlines()), self.cores)
        for c, group in enumerate(groups):
            # print 'group', len(group)
            # print 'c', c
            part = c + 1
            part_writer = open('%s/part.%s.vcf' % (prefix, part), 'w')
            for line in group:
                part_writer.writelines(line)
            part_writer.close()

    #convert and annotate the vcf file to snpeff
    def annotate(self, out_prefix):
        #print 'Hello'
        #print self.dbnfsp_reader
        #header is at:
        
        # 24    SIFT_score: SIFT score (SIFTori).
        # 105 HUVEC_confidence_value: 0 - highly significant scores (approx. p<.003); 1 - significant scores

        #188   clinvar_rs: rs number from the clinvar data set
        #191 clinvar_golden_stars: ClinVar Review Status summary.
        
        func_pred_start = 23
        func_pred_end = 105
        
        clinvar_start = 187
        clinvar_end = 191


        # print 'input',vcffile, out_prefix, dbnsfp 
        dbnfsp_reader = pysam.Tabixfile(settings.dbnsfp, 'r')
        
        # print('header')
        for item in dbnfsp_reader.header:
            header = item.decode('utf-8').strip().split('\t')

        # header = dbnfsp_reader.header.next().strip().split('\t')

        vcffile = 'func_pred/part.%s.vcf' % (out_prefix)

        vcf_reader = open('%s' % (vcffile))
        vcf_writer = open('func_pred/func_pred.%s.vcf' % (out_prefix), 'w')
        
        for line in vcf_reader:
            if line.startswith('#'):
                if line.startswith('#CHROM'):
                    vcf_writer.writelines(dbnfsp_header)
                vcf_writer.writelines(line)
            else:
                variant = line.split('\t')
                variant[0] = variant[0].replace('chr', '')
                index = '%s-%s' % (variant[0], variant[1])
                #print index
                try:
                    records = dbnfsp_reader.fetch(variant[0], int(variant[1])-1, int(variant[1]))
                except:
                    records = []
                    
                for record in records:
                    ann = record.strip().split('\t')

                    ispresent = False
                    if variant[3] == ann[2]:
                        alts = variant[4].split(',')
                        alts_ann = ann[3].split(',')
                        #compare ALT
                        for alt in alts:
                            if alt in alts_ann:
                                ispresent = True

                    if ispresent:
                        new_ann = []
                        
                        for k, item in enumerate(header[func_pred_start:func_pred_end]):
                            idx = k+func_pred_start
                            if ann[idx] != '.':
                                new_ann.append('dbNSFP_%s=%s' % (item, ann[idx].replace(';', '|')))

                        for k, item in enumerate(header[clinvar_start:clinvar_end]):
                            idx = k+clinvar_start
                            if ann[idx] != '.':
                                new_ann.append('dbNSFP_%s=%s' % (item, ann[idx].replace(';', '|').replace(' ', '_')))

                        variant[7] = '%s;%s' % (variant[7], ";".join(new_ann))
                vcf_writer.writelines("\t".join(variant))


if  __name__ == '__main__' :
    
    
    parser = argparse.ArgumentParser(description='Annotate a VCF File with DbNSFP.')

    parser.add_argument('-i', dest='vcf_file', required=True, metavar='example.vcf', help='a VCF file to be annotated')

    parser.add_argument('-n', dest='cores', required=True, metavar='4', help='number of cores to use')

    args = parser.parse_args()

    func_pred = FUNC_PRED_Annotator(args.vcf_file, args.cores)
    func_pred.run()



# def annotate(vcffile, index, dbnsfp):
#     print 'vcf', vcffile, index
#     # filename = os.path.splitext(os.path.basename(str(vcffile)))[0]
#     dbnfsp_reader = pysam.Tabixfile(dbnsfp)

# splitvcf(args.vcffile)

# # job_server = pp.Server()
# prefix = 'func_pred'
# # Define your jobs
# jobs = []
# final_parts = []
# for n in range(0,cores):
#     index = n+1
#     part = '%s/part.%s.vcf' % (prefix, index)
    
#     job = Process(target=annotate, args=(part, index, args.dbnsfp))
#     final_file = 'func_pred/func_pred.%s.vcf' % (index)
#     final_parts.append(final_file)
#     jobs.append(job)


# for job in jobs:
#     job.start()

# for job in jobs:
#     job.join()

# command = 'cat %s/header.vcf ' % (prefix) + " ".join(final_parts) + '> %s/func_pred.vcf' % (prefix)
# os.system(command)
# #merge all files 
